--todo setting to show total refund value in dialogue before confirm
--todo fix that bug where the calculated refund value is different from the actual value, hence the above ^

_G.SellStuff = {}
SellStuff._path = (SellStuffCore and SellStuffCore:GetPath()) or ModPath --if beardlib is installed, use the ModCore:GetPath() method, as it's more reliable than the global ModPath variable
SellStuff._data_path = SavePath .. "SellAllItems_Settings.json" 

SellStuff.selected_category = nil --holds the currently selected category (set by user, resets each game load)
SellStuff.settings = {
	show_total = true,
	can_sell_equipped = false,
	check_equipped_multiprofiles = true --if true, when checking for whether an item is equipped, also checks whether the item is equipped on other loadout profiles
}

--inventory categories
SellStuff.sellable_categories = {
	[1] = "none", --not an inventory category name; items cannot be sold while the category is set to this
	[2] = "primaries",
	[3] = "secondaries",
	[4] = "masks",
	[5] = "all" --not an inventory category name; makes this mod sell items in all categories (primaries, secondaries, and masks)
--			"armors",
--			"grenades",
--			"melee_weapons"
}

--key name conversion for inventory category -> save data item category
SellStuff.multiprofile_categories = {
	primaries = "primary",
	secondaries = "secondary",
	masks = "mask"
}

function SellStuff:ShouldShowSuccessDialog()
	return self.settings.show_total
end

function SellStuff:Load()
	local file = io.open(self._data_path, "r")
	if (file) then
		for k, v in pairs(json.decode(file:read("*all"))) do
			self.settings[k] = v
		end
	else
		SellStuff:Save()
	end
end

function SellStuff:Save()
	local file = io.open(self._data_path,"w+")
	if file then
		file:write(json.encode(self.settings))
		file:close()
	end
end

Hooks:Add("LocalizationManagerPostInit", "LocalizationManagerPostInit_SellStuff", function( loc )
	if not BeardLib then
		loc:load_localization_file( SellStuff._path .. "localization/en.json")
	end
end)

function SellStuff:CanSellEquipped()
	return self.settings.can_sell_equipped
end

function SellStuff:CreateConfirmDialog(params)
	local category = self.sellable_categories[SellStuff.selected_category]
	local title = managers.localization:text("dialog_bm_crafted_sell_title")
	if self:CanSellEquipped() then
		desc = managers.localization:text("sellstuff_sell_dialog_include_equipped_desc")
	else
		desc = managers.localization:text("sellstuff_sell_dialog_exclude_equipped_desc")
	end
	desc = string.format(desc,category)
	local options = {
		{
			text = "Sell it!",
			callback = params.callback
		},
		{
			text = managers.localization:text("dialog_cancel"),
			is_cancel_button = true
		}
	}
	QuickMenu:new(title,desc,options,true)
end

function SellStuff:ShowRefundAmount(count,refund)
	local title,desc
	if count and count > 0 then
		local refund_string = managers.money:add_decimal_marks_to_string(tostring(refund))
		title = managers.localization:text("sellstuff_dialog_success_title")
		desc = managers.localization:text("sellstuff_dialog_success_desc")
		desc = string.gsub(desc,"##COUNT##",count)
		desc = string.gsub(desc,"##REFUND##",refund_string)
	else
		title = managers.localization:text("sellstuff_dialog_fail_title")
		desc = managers.localization:text("sellstuff_dialog_fail_desc")
	end
	QuickMenu:new(title,desc,{},true)
end

function SellStuff:ShouldCheckMultiProfileEquipped()
	return self.settings.check_equipped_multiprofiles
end

function SellStuff:CallbackSellAll()
	local count,refund = self:SellAll()
	if self:ShouldShowSuccessDialog() then
		self:ShowRefundAmount(count,refund)
	end
end

function SellStuff:CallbackSellCategory(category)
	local count,refund = self:SellCategory(category)
	if self:ShouldShowSuccessDialog() then
		self:ShowRefundAmount(count,refund)
	end
end

function SellStuff:SellAll()
	local total_refund = 0
	local total_items = 0
	for _,cat in pairs({"primaries","secondaries","masks"}) do
		local _total_items,_total_refund = self:SellCategory(cat)
		total_refund = total_refund + _total_refund
		total_items = total_items + _total_items
	end
	return total_items,total_refund
end

function SellStuff:SellCategory(category)
	local total_items = 0
	local total_refund = 0
	local bm = managers.blackmarket
	local mm = managers.money
	local mpm = managers.multi_profile
	for slot,data in pairs(bm._global.crafted_items[category]) do
		local is_equipped = data.equipped
		if not is_equipped and self:ShouldCheckMultiProfileEquipped() then
			--check whether this item is equipped on other profiles
			for profile_index,profile_data in ipairs(mpm._global._profiles) do 
				local profile_category = self.multiprofile_categories[category]
				if profile_category and profile_data[profile_category] == slot then
					is_equipped = true
					break
				end
			end
		end
		
		if self:CanSellEquipped() or not is_equipped then
			local id
			local value
			if category == "masks" then
				if data.mask_id ~= "character_locked" then
					id = data.mask_id
					value = mm:get_mask_slot_sell_value(slot)
					bm:on_sell_mask(slot,true)
				end
			else --is weapon ("primaries" or "secondaries")
				id = data.weapon_id
				value = mm:get_weapon_slot_sell_value(category,slot)
				bm:on_sell_weapon(category,slot,true)
			end
			
			if id then
				total_items = total_items + 1
			end
			if value then
				total_refund = total_refund + value
			end
		end
		
	end
	
	return total_items,total_refund
end

Hooks:Add( "MenuManagerInitialize", "MenuManagerInitialize_SellStuff", function(menu_manager)
		
	MenuCallbackHandler.callback_sellstuff_selectcategory = function(self,item)
		SellStuff.selected_category = tonumber(item:value())
	end
	MenuCallbackHandler.callback_sellstuff_sellselected = function(self,item)
		if SellStuff.selected_category then
			if not Utils:IsInHeist() then
				local category = SellStuff.sellable_categories[SellStuff.selected_category]
				if category == "all" then 
					SellStuff:CreateConfirmDialog({callback = callback(SellStuff,SellStuff,"CallbackSellAll")})
				elseif category ~= "none" then
					SellStuff:CreateConfirmDialog({callback = callback(SellStuff,SellStuff,"CallbackSellCategory",category)})
				end
			else
				--QuickMenu:new(managers.localization:text("sellstuff_cannot_sell_midheist_title"),managers.localization:text("sellstuff_cannot_sell_midheist_desc"),{},true)
			end
		end
	end

	MenuCallbackHandler.callback_sellstuff_can_sell_equipped = function(self,item)
		SellStuff.settings.can_sell_equipped = item:value() == "on"
		SellStuff:Save()
	end
	
	MenuCallbackHandler.callback_sellstuff_should_show_success_dialog = function(self,item)
		SellStuff.settings.show_total = item:value() == "on"
		SellStuff:Save()
	end
	
	MenuCallbackHandler.callback_sellstuff_check_equipped_multiprofiles = function(self,item)
		SellStuff.settings.check_equipped_multiprofiles = item:value() == "on"
		SellStuff:Save()
	end
	
	MenuCallbackHandler.callback_sellstuff_close = function(self)
--		SellStuff:Save()
	end
	SellStuff:Load()
	MenuHelper:LoadFromJsonFile(SellStuff._path .. "menu/options.json", SellStuff, SellStuff.settings)
end)
